==> eh-0.4.tar.gz: eh-0.4/src/ehfiltrk.c <==
/***

ehfiltrk.c -- Error Handler, file stream tracking (public interface)

Copyright (c) 1992-6 Christopher Cobb, All rights reserved.

***/

static const char*rcsid =
       "$Id: ehfiltrk.c,v 1.7 1996/03/08 07:35:10 ccobb Exp $";

#include "eh.h"
#include "eherrs.h"

EHBEGINEXTERNC
#include <string.h>
EHENDEXTERNC

#include "ehbounds.h"
#include "ehintres.h"

#define EH_NFILTRK 1
#include "ehfiltrk.h"

EHFILENAME("ehfiltrk.c");

/***
_ehFopen()
Track files opened with fopen().
This function is an fopen() wrapper.  
***/

FILE *
_ehFopen(
   const char *pcFile,
   const char *pcFunc,
   int         iLine,
   const char *pcFileName,
   const char *pcMode)
{
   FILE *volatile pFile = NULL;
   volatile bool completed = false;

   EHCATCH(_ehFopen) {

      pFile = fopen(pcFileName, pcMode);
      if (!pFile)
      {
         ehPError(pcFile, pcFunc, iLine, (schar)1, EH_FOPEN);
         break;
      }

      //fprintf(EHSTDERR, "_ehFopen: pFile = %p\n", pFile);
      completed = ehResObt(pcFile,pcFunc,iLine,&ehIntRes,
            (int)EH_RES_FILESTR, pcFileName,
            (pfv_ulv)fclose, 1L, 
            (ulong)pFile);

   } EHENDCATCH;

   if (ehError() && !completed)
   {
      if (pFile)
      {
         fclose(pFile);
         pFile = NULL;
      }
   }

   EHRETURN(pFile);
} /* _ehFopen */


/***
_ehTmpFile()
Track files opened with tmpfile().
***/

FILE *ehTmpFile A((const char *pcFil, const char *pcFunc, int iLin));
FILE *
_ehTmpFile(
   const char *pcFile,
   const char *pcFunc,
   int         iLine)
{
   FILE *volatile pFile = NULL;
   volatile bool completed = false;

   EHCATCH(_ehTmpFile) {

      pFile = tmpfile();
      if (!pFile)
      {
         ehPError(pcFile, pcFunc, iLine, (schar)-1, EH_TMPFILE);
         break;
      }

      //fprintf(EHSTDERR, "_ehTmpFile: pFile = %p\n", pFile);
      completed = ehResObt(pcFile,pcFunc,iLine,&ehIntRes,
            (int)EH_RES_FILESTR, "temporary file",
            (pfv_ulv)fclose, 1L, 
            (ulong)pFile);
            //(pfv_ulv)fclose, 0L);

   } EHENDCATCH;

   if (ehError() && !completed)
   {
      if (pFile)
      {
         fclose(pFile);
         pFile = NULL;
      }
   }

   EHRETURN(pFile);
} /* _ehTmpFile */


/***
_ehFreopen()
Track files opened with freopen().
This function is an freopen() wrapper.  
***/

FILE *
_ehFreopen(
   const char *pcFile,
   const char *pcFunc,
   int         iLine,
   const char *pcFileName,
   const char *pcMode,
   FILE       *pFileOld)
{
   FILE *volatile pFileNew = NULL;
   volatile bool completed = false;

   EHCATCH(_ehFreopen) {

   /***

      We need to check against stdin, stdout and stderr
      which might be reopened but are not already in 
      the resource tree.

      If the reopened file is stdin, stdout or stderr,
      then don't call ehResReObt().
      Call ehResObt() instead since we are not
      re-obtaining a resource that is in the resource
      tree.  However, if the same resource handle is
      returned (e.g., stderr), then we may have a
      problem because next time through, stderr will
      be a resource in the tree, but we won't know it.

   ***/

      /* attempt to reopen the file */
      pFileNew = freopen(pcFileName, pcMode, pFileOld);
      if (pFileNew == NULL)
      {
         ehPError(pcFile, pcFunc, iLine, (schar)1, EH_REALLOC);
         break;
      }

      if (pFileOld != stdin && pFileOld != stdout && pFileOld != stderr)
      {
         /* mark the resource as obtained */
         //fprintf(EHSTDERR, "_ehFreopen: pFileNew = %p\n", pFileNew);
         completed = ehResReObt(pcFile,pcFunc,iLine,&ehIntRes,
               (int)EH_RES_FILESTR, pcFileName,
               (ulong)pFileOld,
               (pfv_ulv)free, 1L, 
               (ulong)pFileNew);
      }

   } EHENDCATCH;

   if (ehError() && !completed)
   {
      if (pFileNew && pFileNew != pFileOld)
      {
         fclose(pFileNew); /* close the new file */
         pFileNew = NULL;
         /* now both files are closed */
      }
      /* mark the old file as closed */
      ehResRel(pcFile,pcFunc,iLine,&ehIntRes,
            (int)EH_RES_FILESTR, (pfv_ulv)0, 1L,
            (ulong)pFileOld);
   }
   EHRETURN(pFileNew);
} /* _ehFreopen */


/***
_ehFclose()
Track heap file stream closings.
***/

void
_ehFclose(
   const char *pcFile,
   const char *pcFunc,
   int         iLine,
   FILE       *pFile)
{
   EHCATCH(_ehFclose) {

   } EHENDCATCH;

   if (pFile != stdin && pFile != stdout && pFile != stderr)
   {
      /* mark the resource and release it */
      //fprintf(EHSTDERR, "_ehFclose: pFile = %p\n", pFile);
      ehResRel(pcFile,pcFunc,iLine,&ehIntRes,
            (int)EH_RES_FILESTR, (pfv_ulv)fclose, 1L, 
            (ulong)pFile);
   }

   EHLEAVE();
} /* _ehFclose */

==> eh-0.5.tar.gz: eh-0.5/src/ehfiltrk.c <==
/***

ehfiltrk.c -- Error Handler, file stream tracking (public interface)

    Copyright (C) 1997  CSEG, Inc., All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   

    The GNU General Public License is in the file COPYING which resides
    in the top directory of the distribution tree.

***/

static const char*rcsid =
       "$Id: ehfiltrk.c,v 1.7 1996/03/08 07:35:10 ccobb Exp $";

#define EH_NO_FILE_TRACKING 1
#include "ehfiltrk.h"
#include "eherrs.h"

EHBEGINEXTERNC
#include <string.h>
EHENDEXTERNC

#include "ehbounds.h"
#include "ehintres.h"

EHFILENAME("ehfiltrk.c");

/***
_ehFopen()
Track files opened with fopen().
This function is an fopen() wrapper.  
***/

FILE *
_ehFopen(
   const char *pcFile,
   const char *pcFunc,
   int         iLine,
   const char *pcFileName,
   const char *pcMode)
{
   FILE *volatile pFile = NULL;
   volatile bool completed = false;

   EHTRY(_ehFopen) {

      pFile = fopen(pcFileName, pcMode);
      if (!pFile)
      {
         ehPError(pcFile, pcFunc, iLine, (schar)1, EH_FOPEN);
         break;
      }

      //fprintf(EHSTDERR, "_ehFopen: pFile = %p\n", pFile);
      completed = ehResObt(pcFile,pcFunc,iLine,&ehResTrk,
            (int)EH_RES_FILESTR, pcFileName,
            (pfv_ulv)fclose, 1L, 
            (ulong)pFile);

   } EHCATCH;

   if (ehError() && !completed)
   {
      if (pFile)
      {
         fclose(pFile);
         pFile = NULL;
      }
   }

   EHRETURN(pFile);
} /* _ehFopen */


/***
_ehTmpFile()
Track files opened with tmpfile().
***/

FILE *ehTmpFile A((const char *pcFil, const char *pcFunc, int iLin));
FILE *
_ehTmpFile(
   const char *pcFile,
   const char *pcFunc,
   int         iLine)
{
   FILE *volatile pFile = NULL;
   volatile bool completed = false;

   EHTRY(_ehTmpFile) {

      pFile = tmpfile();
      if (!pFile)
      {
         ehPError(pcFile, pcFunc, iLine, (schar)-1, EH_TMPFILE);
         break;
      }

      //fprintf(EHSTDERR, "_ehTmpFile: pFile = %p\n", pFile);
      completed = ehResObt(pcFile,pcFunc,iLine,&ehResTrk,
            (int)EH_RES_FILESTR, "temporary file",
            (pfv_ulv)fclose, 1L, 
            (ulong)pFile);
            //(pfv_ulv)fclose, 0L);

   } EHCATCH;

   if (ehError() && !completed)
   {
      if (pFile)
      {
         fclose(pFile);
         pFile = NULL;
      }
   }

   EHRETURN(pFile);
} /* _ehTmpFile */


/***
_ehFreopen()
Track files opened with freopen().
This function is an freopen() wrapper.  
***/

FILE *
_ehFreopen(
   const char *pcFile,
   const char *pcFunc,
   int         iLine,
   const char *pcFileName,
   const char *pcMode,
   FILE       *pFileOld)
{
   FILE *volatile pFileNew = NULL;

   EHTRY(_ehFreopen) {

      /* attempt to reopen the file */
      pFileNew = freopen(pcFileName, pcMode, pFileOld);
      if (pFileNew == NULL)
      {
         ehPError(pcFile, pcFunc, iLine, (schar)1, EH_REALLOC);
         break;
      }

      /* mark the resource as obtained */
      //fprintf(EHSTDERR, "_ehFreopen: pFileNew = %p\n", pFileNew);
      ehResReObt(pcFile,pcFunc,iLine,&ehResTrk,
            (int)EH_RES_FILESTR, pcFileName,
            (ulong)pFileOld,
            (pfv_ulv)free, 1L, 
            (ulong)pFileNew);

   } EHCATCH;

   if (ehError())
   {
      if (pFileNew && pFileNew != pFileOld)
      {
         fclose(pFileNew); /* close the new file */
         pFileNew = NULL;
         /* now both files are closed */
      }
      /* mark the old file as closed */
      ehResRel(pcFile,pcFunc,iLine,&ehResTrk,
            (int)EH_RES_FILESTR, (pfv_ulv)0, 1L,
            (ulong)pFileOld);
   }
   EHRETURN(pFileNew);
} /* _ehFreopen */


/***
_ehFclose()
Track heap file stream closings.
***/

void
_ehFclose(
   const char *pcFile,
   const char *pcFunc,
   int         iLine,
   FILE       *pFile)
{
   EHTRY(_ehFclose) { } EHCATCH;

   /* mark the resource and release it */
   //fprintf(EHSTDERR, "_ehFclose: pFile = %p\n", pFile);
   ehResRel(pcFile,pcFunc,iLine,&ehResTrk,
         (int)EH_RES_FILESTR, (pfv_ulv)fclose, 1L, 
         (ulong)pFile);

   EHLEAVE();
} /* _ehFclose */
